package DE_EPAGES::Product::t::ProductResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 14;

use Benchmark qw( timethese );
use Data::Dumper;
use HTTP::Status qw( HTTP_BAD_REQUEST HTTP_METHOD_NOT_ALLOWED HTTP_NOT_FOUND );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::Product;
use DE_EPAGES::WebInterface::API::FakeRequest;
use DE_EPAGES::WebService::API::Codec::JSON;
use DE_EPAGES::WebService::API::JSONRestClient;

## no critic qw( ProhibitMagicNumbers );

sub test {
    my $self = shift;

    $self->test_get;
    $self->test_get_v2;
    $self->test_put;
    $self->testBenchmark;
}

sub _basicAuthentication {
    my $self = shift;

    return [ $self->shop->get('SuperMerchant')->alias, 'admin' ];
}

sub _resource {
    my $self = shift;
    my ($ContentType) = @_;

    return DE_EPAGES::WebService::API::JSONRestClient->new('http://test-domain/api/products', {
        UserAgent => DE_EPAGES::WebInterface::API::FakeRequest->new,
        BasicAuthentication => $self->_basicAuthentication,
        Codec => DE_EPAGES::WebService::API::Codec::JSON->new(
            ContentType => $ContentType
        ),
    });
}

sub _productGUID {
    my $self = shift;
    my ($Alias) = @_;

    return $self->shop->get('ProductFolder')->child($Alias)->guid;
}

sub _TestHttpError(&$$) {
    my ( $Sub, $ExpectedStatus, $Message ) = @_;

    eval { $Sub->() };
    ok ExistsError(), "$Message: error expected";
    my $Error = GetError();
    is $Error->code, 'HttpError', "$Message: error code";
    is $Error->vars->{'Code'}, $ExpectedStatus, "$Message: HTTP status code=$ExpectedStatus"
}

sub test_get {
    my $self = shift;

    my $Resource = $self->_resource('application/json');
    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';
    my $ProductGUID = $self->_productGUID('MyPetrol');

    my $hData = $Resource->get( $ProductGUID ."?locale=$LocaleID&currency=$CurrencyID" );

    # details are tested by ProductResource.t
    is( scalar keys %$hData, 20, 'GET MyPetrol v1' ) or diag Dumper($hData);

    _TestHttpError sub { $Resource->get( 'UnknownGUID?locale=en_GB&currency=EUR' ) },
        HTTP_NOT_FOUND,
        'invalid guid - Not Found';

    _TestHttpError sub { $Resource->get( $ProductGUID .'?locale=de_DE&currency=EUR' ) },
        HTTP_BAD_REQUEST,
        'invalid locale - Bad Request';

    _TestHttpError sub { $Resource->get( $ProductGUID .'?locale=en_GB&currency=DEM' ) },
        HTTP_BAD_REQUEST,
        'invalid currency - Bad Request';
}

sub test_put {
    my $self = shift;

    my $Resource = $self->_resource('application/json');
    my $ProductGUID = $self->_productGUID('MyPetrol');

    _TestHttpError sub { $Resource->put( $ProductGUID ) },
        HTTP_METHOD_NOT_ALLOWED,
        'invalid request method (PUT)';
}

sub test_get_v2 {
    my $self = shift;

    my $Resource = $self->_resource('application/vnd.epages.product+json; version=2.0');
    my $LocaleID = 'de_DE';
    my $CurrencyID = 'EUR';
    my $TaxModel = 'gross';
    my $ProductGUID = $self->_productGUID('MyPetrol');

    my $hData = $Resource->get( $ProductGUID ."?locale=$LocaleID&currency=$CurrencyID&tax-model=$TaxModel" );

    is( scalar keys %$hData, 8, 'GET MyPetrol v2' ) or diag Dumper($hData);
}

sub testBenchmark {
    my $self = shift;

    my $Resource1 = $self->_resource('application/json');
    my $Resource2 = $self->_resource('application/vnd.epages.product+json; version=2.0');
    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';
    my $ProductGUID = $self->_productGUID('MyPetrol');
    my $TaxModel = 'gross';

    timethese(200, {
        v1 => sub { $Resource1->get( "$ProductGUID?locale=$LocaleID&currency=$CurrencyID" ); },
        v2 => sub { $Resource2->get( "$ProductGUID?locale=$LocaleID&currency=$CurrencyID&tax-model=$TaxModel" ); },
    });
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);

    $self->shop->set({DomainName => 'test-domain'});
}

__PACKAGE__->runScript;
